"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const R = require("ramda");
const semver = require("semver");
function mergeLockfile(opts) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v;
    const newLockfile = {
        importers: {},
        lockfileVersion: Math.max(opts.base.lockfileVersion, opts.ours.lockfileVersion),
    };
    for (const importerId of Array.from(new Set([...Object.keys(opts.ours.importers), ...Object.keys(opts.theirs.importers)]))) {
        newLockfile.importers[importerId] = {
            specifiers: {},
        };
        for (const key of ['dependencies', 'devDependencies', 'optionalDependencies']) {
            newLockfile.importers[importerId][key] = mergeDict((_b = (_a = opts.ours.importers[importerId]) === null || _a === void 0 ? void 0 : _a[key]) !== null && _b !== void 0 ? _b : {}, (_d = (_c = opts.base.importers[importerId]) === null || _c === void 0 ? void 0 : _c[key]) !== null && _d !== void 0 ? _d : {}, (_f = (_e = opts.theirs.importers[importerId]) === null || _e === void 0 ? void 0 : _e[key]) !== null && _f !== void 0 ? _f : {}, key, mergeVersions);
            if (!Object.keys(newLockfile.importers[importerId][key]).length) {
                delete newLockfile.importers[importerId][key];
            }
        }
        newLockfile.importers[importerId].specifiers = mergeDict((_h = (_g = opts.ours.importers[importerId]) === null || _g === void 0 ? void 0 : _g.specifiers) !== null && _h !== void 0 ? _h : {}, (_k = (_j = opts.base.importers[importerId]) === null || _j === void 0 ? void 0 : _j.specifiers) !== null && _k !== void 0 ? _k : {}, (_m = (_l = opts.theirs.importers[importerId]) === null || _l === void 0 ? void 0 : _l.specifiers) !== null && _m !== void 0 ? _m : {}, 'specifiers', takeChangedValue);
    }
    const packages = {};
    for (const depPath of Array.from(new Set([...Object.keys((_o = opts.ours.packages) !== null && _o !== void 0 ? _o : {}), ...Object.keys((_p = opts.theirs.packages) !== null && _p !== void 0 ? _p : {})]))) {
        const basePkg = (_q = opts.base.packages) === null || _q === void 0 ? void 0 : _q[depPath];
        const ourPkg = (_r = opts.ours.packages) === null || _r === void 0 ? void 0 : _r[depPath];
        const theirPkg = (_s = opts.theirs.packages) === null || _s === void 0 ? void 0 : _s[depPath];
        const pkg = {
            ...basePkg,
            ...ourPkg,
            ...theirPkg,
        };
        for (const key of ['dependencies', 'optionalDependencies']) {
            pkg[key] = mergeDict((_t = ourPkg === null || ourPkg === void 0 ? void 0 : ourPkg[key]) !== null && _t !== void 0 ? _t : {}, (_u = basePkg === null || basePkg === void 0 ? void 0 : basePkg[key]) !== null && _u !== void 0 ? _u : {}, (_v = theirPkg === null || theirPkg === void 0 ? void 0 : theirPkg[key]) !== null && _v !== void 0 ? _v : {}, key, mergeVersions);
            if (!Object.keys(pkg[key]).length) {
                delete pkg[key];
            }
        }
        packages[depPath] = pkg;
    }
    newLockfile.packages = packages;
    return newLockfile;
}
exports.default = mergeLockfile;
function mergeDict(ourDict, baseDict, theirDict, fieldName, valueMerger) {
    const newDict = {};
    for (const key of R.keys(ourDict).concat(R.keys(theirDict))) {
        const changedValue = valueMerger(ourDict[key], baseDict[key], theirDict[key], `${fieldName}.${key}`);
        if (changedValue) {
            newDict[key] = changedValue;
        }
    }
    return newDict;
}
function takeChangedValue(ourValue, baseValue, theirValue, fieldName) {
    if (ourValue === theirValue)
        return ourValue;
    if (baseValue === ourValue)
        return theirValue;
    if (baseValue === theirValue)
        return ourValue;
    // eslint-disable-next-line
    throw new Error(`Cannot resolve '${fieldName}'. Base value: ${baseValue}. Our: ${ourValue}. Their: ${theirValue}`);
}
function mergeVersions(ourValue, baseValue, theirValue, fieldName) {
    if (ourValue === theirValue)
        return ourValue;
    if (baseValue === ourValue)
        return theirValue;
    if (baseValue === theirValue)
        return ourValue;
    const [ourVersion] = ourValue.split('_');
    const [theirVersion] = theirValue.split('_');
    if (semver.gt(ourVersion, theirVersion)) {
        return ourValue;
    }
    return theirValue;
}
//# sourceMappingURL=index.js.map